unit Unit1;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, VclTee.TeeGDIPlus, Vcl.ExtCtrls,
  VCLTee.TeEngine, VCLTee.TeeProcs, VCLTee.Chart, Vcl.StdCtrls;

type
  TForm1 = class(TForm)
    Image1: TImage;
    Panel1: TPanel;
    chkCustomDrawSymbol: TCheckBox;
    procedure chkCustomDrawSymbolClick(Sender: TObject);
  private
    procedure CreateGapImage(const aBitmap: TBitmap);
    procedure GapChartAfterDraw(Sender: TObject);
    procedure GapChartLegendDrawSymbol(Sender: TObject; Series: TChartSeries; ValueIndex: Integer; R: TRect);
    function CreateChart: TChart;
    procedure ChartAxisLabel (Sender: TChartAxis; Series: TChartSeries; ValueIndex: Integer; var LabelText: String);
    procedure AdjustGapChartLegend(aChart: TChart);
    procedure UpdateChart;
    { Private declarations }
  protected
    procedure Resize; override;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

uses
  Series, TeCanvas, math;

{$R *.dfm}

procedure TForm1.ChartAxisLabel(Sender: TChartAxis; Series: TChartSeries;
  ValueIndex: Integer; var LabelText: String);
var
  lPos: Integer;
begin
  if (Sender.ParentChart.LeftAxis = Sender) and (Sender.Maximum >= 10000) then
  begin
    lPos := Pos(',', LabelText);
    if lPos > 0 then
      LabelText := Copy(LabelText, 1, lPos-1) + 'K';
  end;
end;

function TForm1.CreateChart: TChart;
begin
  result := TChart.Create(nil);

  result.Color := clWhite;
  result.BevelOuter := bvNone;
  result.LeftAxis.Grid.Color := clSilver;

  result.LeftAxis.LabelsFont.Name := 'Arial Narrow'; {added from Gap Chart}
  result.LeftAxis.AutomaticMinimum := False; {added from Gap Chart}
  result.LeftAxis.Minimum := 0; {added from Gap Chart}
  result.LeftAxis.MinorTicks.Visible := False; {added from Gap Chart}
  result.LeftAxis.TicksInner.Visible := False; {added from Gap Chart}
  result.LeftAxis.Ticks.Style := psDot;
  result.LeftAxis.TickLength := 2; {added from Gap Chart}
  result.LeftAxis.Axis.Width := 1;
  result.LeftAxis.Title.Font.Name := 'Arial Narrow';

  result.BottomAxis.Axis.Color := clSilver;
  result.BottomAxis.MinorTicks.Visible := false; {added from Gap Chart}
  result.BottomAxis.Ticks.Visible := false;
  result.BottomAxis.LabelsFont.Name := 'Arial Narrow';
  result.BottomAxis.Grid.Visible := False;
  result.BottomAxis.Axis.Width := 1;

  result.Legend.Frame.Visible := False;
  result.Legend.Title.Font.Name := 'Arial Narrow';
  result.Legend.Title.Font.Style := [];
  result.Legend.Shadow.VertSize := 0;
  result.Legend.LegendStyle := lsSeries;
  result.Legend.Shadow.HorizSize := 0;
  result.Legend.Alignment := laRight;
  result.Legend.Transparent := True;

  result.LeftAxis.Axis.Color := clSilver;
  result.LeftAxis.AxisValuesFormat := '$,0';

  result.Walls.Visible := False;

  result.BackWall.Pen.Color := clSilver;
  result.OnGetAxisLabel := ChartAxisLabel;

  result.View3D := False;
end;


procedure TForm1.CreateGapImage(const aBitmap: TBitmap);
var
  lChart: TChart;

  function CreateChartSeries(const aIndex: integer): TAreaSeries;
  var
    lCreateSrs: boolean;
  begin
    result := TAreaSeries.Create(self);
    result.ParentChart := lChart;
    result.AreaLinesPen.Visible := false;
    result.Color := clRed;
    result.Pen.Visible := false;
    result.FillSampleValues(10);
    if (aIndex = 1) then
      result.Transparency := 50;
  end;

const
  SCALE_FACTOR = 1;
var
  lRect: TRect;
  lsrs: TChartSeries;
  I: Integer;
  J: Integer;
  lIncome: Double;
  lLastIdx: Integer;
  lMonthlyIncomes: array of double;
  lBmp: TBitmap;
begin

  lChart := CreateChart;
  try
    lChart.OnGetAxisLabel := ChartAxisLabel;
    lChart.LeftAxis.Title.Caption := 'Annual Income';
    lChart.LeftAxis.Title.Font.Size := 10;
    lChart.BottomAxis.Title.Caption := 'Years In Retirement';

    lChart.BottomAxis.Title.Font.Size := 10;
    lChart.Legend.CustomPosition := True;
    if chkCustomDrawSymbol.Checked then
      lChart.Legend.Symbol.OnDraw := GapChartLegendDrawSymbol; {$message warn 'TdmBPPrinting.CreateGapImage: Remove when legend drawing is fixed. [KNOWN ISSUES]'};
    lChart.OnAfterDraw := GapChartAfterDraw;


    for I := 1 to 1 do
      lsrs := CreateChartSeries(I);


    lChart.Legend.Font.Size := Ceil(lChart.Legend.Font.Size * SCALE_FACTOR);
    lChart.BottomAxis.LabelsFont.Size := Ceil(lChart.BottomAxis.LabelsFont.Size * SCALE_FACTOR);
    lChart.BottomAxis.Title.Font.Size := Ceil(lChart.BottomAxis.Title.Font.Size * SCALE_FACTOR);
    lChart.LeftAxis.LabelsFont.Size := Ceil(lChart.LeftAxis.LabelsFont.Size * SCALE_FACTOR);
    lChart.LeftAxis.Title.Font.Size := Ceil(lChart.LeftAxis.Title.Font.Size * SCALE_FACTOR);

    lRect := Rect(0, 0, aBitmap.Width * SCALE_FACTOR, aBitmap.Height * SCALE_FACTOR);

    lChart.SetBounds(0, 0, lRect.Width, lRect.Height);

    lChart.Draw;

    lBmp := lChart.TeeCreateBitmap(clWhite, lRect);
    try
      aBitmap.Assign(lBmp);
    finally
      lBmp.Free;
    end;

  finally // wrap up
    lChart.FreeAllSeries;
    lChart.Free;
  end;    // try/finally
end;


procedure TForm1.GapChartAfterDraw(Sender: TObject);
begin
  Assert(sender is TChart);
  AdjustGapChartLegend(Sender as TChart);
end;

procedure TForm1.AdjustGapChartLegend(aChart: TChart);
var
  lLeft: Integer;
begin
  if (aChart.SeriesCount > 0) and (aChart.Series[0].Count > 0) then
  begin
    lLeft := aChart.Series[0].CalcXPos(1);
    aChart.Legend.Left := lLeft;
  end;
end;

procedure TForm1.chkCustomDrawSymbolClick(Sender: TObject);
begin
  UpdateChart;
end;

procedure TForm1.GapChartLegendDrawSymbol(Sender: TObject; Series: TChartSeries;
  ValueIndex: Integer; R: TRect);
var
  lCanvas: TCanvas3D;
  lBlend: TTeeBlend;
  lAreaSeries: TAreaSeries;
begin
  Assert(Series is TAreaSeries);

  lAreaSeries := TAreaSeries(Series);
  lCanvas := Series.ParentChart.Canvas;
  lCanvas.Brush.Color := Series.Color;

  if lAreaSeries.Transparency > 0 then
  begin
    lBlend := lCanvas.BeginBlending(r, lAreaSeries.Transparency);
    try
      lCanvas.Rectangle(R);
    finally
      lCanvas.EndBlending(lBlend);
    end;
  end
  else
    lCanvas.Rectangle(R);
end;

procedure TForm1.UpdateChart;
var
  lBmp: TBitmap;
begin
  lBmp := TBitmap.Create;
  try
    lbmp.SetSize(Image1.Width, Image1.Height);
    CreateGapImage(lbmp);

    Image1.Picture.Bitmap := lbmp;
  finally
    lbmp.Free;
  end;
end;
procedure TForm1.Resize;
begin
  inherited;
  UpdateChart;
end;

end.
